<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use FiloBlu\CegidY2\Api\LoyaltyWcfService\ServiceType\Get;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetAvailableLoyaltyPointsResponse;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetCustomerAvailableLoyaltyPointsResponse;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetCustomerCards;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetCustomerCardsFactory;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetCustomerCardsRequest;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetCustomerCardsRequestFactory;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetCustomerCardsResponse;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetLoyaltyCardResponse;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\GetLoyaltyPointsForSaleResponse;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\RetailContext;
use FiloBlu\CegidY2\Api\LoyaltyWcfService\StructType\RetailContextFactory;
use FiloBlu\CegidY2\Service\CredentialProvider;
use FiloBlu\CegidY2\Service\ServiceProvider;
use Magento\Customer\Api\Data\CustomerInterface;
use RuntimeException;

/**
 * Class Loyalty
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class Loyalty
{
    /**
     * @var ServiceProvider
     */
    protected $getFactory;
    /**
     * @var GetCustomerCardsFactory
     */
    protected $getCustomerCardsFactory;
    /**
     * @var RetailContextFactory
     */
    protected $retailContextFactory;
    /**
     * @var CredentialProvider
     */
    protected $credentialProvider;
    /**
     * @var GetCustomerCardsRequestFactory
     */
    protected $customerCardsRequestFactory;

    /**
     * Loyalty constructor.
     * @param CredentialProvider $credentialProvider
     * @param GetCustomerCardsFactory $getCustomerCardsFactory
     * @param GetCustomerCardsRequestFactory $customerCardsRequestFactory
     * @param RetailContextFactory $retailContextFactory
     * @param ServiceProvider $getFactory
     */
    public function __construct(
        CredentialProvider $credentialProvider,
        GetCustomerCardsFactory $getCustomerCardsFactory,
        GetCustomerCardsRequestFactory $customerCardsRequestFactory,
        RetailContextFactory $retailContextFactory,
        ServiceProvider $getFactory)
    {
        $this->credentialProvider = $credentialProvider;
        $this->getFactory = $getFactory;
        $this->getCustomerCardsFactory = $getCustomerCardsFactory;
        $this->retailContextFactory = $retailContextFactory;
        $this->customerCardsRequestFactory = $customerCardsRequestFactory;
    }

    /**
     * @param CustomerInterface $customer
     * @return GetAvailableLoyaltyPointsResponse|GetCustomerAvailableLoyaltyPointsResponse|GetCustomerCardsResponse|GetLoyaltyCardResponse|GetLoyaltyPointsForSaleResponse
     */
    public function getCustomerCards(CustomerInterface $customer)
    {
        /** @var Get $getService */
        $getService = $this->getFactory->create();
        /** @var GetCustomerCards $parameters */
        $parameters = $this->getCustomerCardsFactory->create();

        /** @var GetCustomerCardsRequest $cardRequest */
        $cardRequest = $this->customerCardsRequestFactory->create();

        $cardRequest->setCustomerId($customer->getCustomAttribute('cegid_id'));
        $parameters->setCustomerCardsRequest($cardRequest);

        $parameters->setClientContext($this->getRetailContext());

        if ($getService->GetCustomerCards($parameters) !== false) {
            return $getService->getResult();
        }

        throw new RuntimeException(implode("\n", $getService->getLastError()));
    }

    /**
     * @return RetailContext
     */
    public function getRetailContext()
    {
        return $this->retailContextFactory->create(['databaseId' =>
            $this->credentialProvider->getDatabase('loyalty_wcf_service')
        ]);
    }
}