<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use FiloBlu\Cegid\Api\SellerRepositoryInterface;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_Line;
use FiloBlu\CegidY2\Api\SaleDocumentService\ArrayType\ArrayOfCreate_LineFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\DocumentOrigin;
use FiloBlu\CegidY2\Api\SaleDocumentService\EnumType\SaleDocumentType;
use FiloBlu\CegidY2\Api\SaleDocumentService\ServiceType\Create;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Header;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_HeaderFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_LineFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_Request;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create_RequestFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\CreateFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\CreateResponse;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\ItemIdentifierFactory;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContext;
use FiloBlu\CegidY2\Api\SaleDocumentService\StructType\RetailContextFactory;
use FiloBlu\CegidY2\Service\CredentialProvider;
use FiloBlu\CegidY2\Service\ServiceProvider;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Customer\Api\Data\CustomerInterfaceFactory;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\Exception\LocalizedException as LocalizedExceptionAlias;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Sales\Api\Data\OrderInterface;
use RuntimeException;

/**
 * Class Order
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class Order
{
    /**
     * @var CredentialProvider
     */
    protected $credentialProvider;
    /**
     * @var RetailContextFactory
     */
    protected $retailContextFactory;
    /**
     * @var ServiceProvider
     */
    protected $createService;
    /**
     * @var CreateFactory
     */
    protected $createFactory;
    /**
     * @var Create_RequestFactory
     */
    protected $createRequestFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var Create_HeaderFactory
     */
    protected $createHeaderFactory;
    /**
     * @var ArrayOfCreate_LineFactory
     */
    protected $arrayOfCreateLineFactory;
    /**
     * @var Create_LineFactory
     */
    protected $createLineFactory;
    /**
     * @var CartRepositoryInterface
     */
    protected $quoteRepository;
    /**
     * @var Customer
     */
    protected $customerHelper;
    /**
     * @var ItemIdentifierFactory
     */
    protected $itemIdentifierFactory;
    /**
     * @var CustomerInterfaceFactory
     */
    private $customerFactory;
    /**
     * @var SellerRepositoryInterface
     */
    private $sellerRepository;
    /**
     * @var SearchCriteriaBuilderFactory
     */
    private $searchCriteriaBuilderFactory;

    /**
     * Order constructor.
     * @param CredentialProvider $credentialProvider
     * @param CreateFactory $createFactory
     * @param CustomerRepositoryInterface $customerRepository
     * @param Create_HeaderFactory $createHeaderFactory
     * @param RetailContextFactory $retailContextFactory
     * @param Create_RequestFactory $createRequestFactory
     * @param Create_LineFactory $createLineFactory
     * @param ArrayOfCreate_LineFactory $arrayOfCreateLineFactory
     * @param ItemIdentifierFactory $itemIdentifierFactory
     * @param CartRepositoryInterface $quoteRepository
     * @param ServiceProvider $createService
     * @param CustomerInterfaceFactory $customerFactory
     * @param SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
     * @param SellerRepositoryInterface $sellerRepository
     * @param Customer $customerHelper
     */
    public function __construct(CredentialProvider $credentialProvider,
                                CreateFactory $createFactory,
                                CustomerRepositoryInterface $customerRepository,
                                Create_HeaderFactory $createHeaderFactory,
                                RetailContextFactory $retailContextFactory,
                                Create_RequestFactory $createRequestFactory,
                                Create_LineFactory $createLineFactory,
                                ArrayOfCreate_LineFactory $arrayOfCreateLineFactory,
                                ItemIdentifierFactory $itemIdentifierFactory,
                                CartRepositoryInterface $quoteRepository,
                                ServiceProvider $createService,
                                CustomerInterfaceFactory $customerFactory,
                                SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory,
                                SellerRepositoryInterface $sellerRepository,
                                Customer $customerHelper)
    {
        $this->credentialProvider = $credentialProvider;
        $this->createFactory = $createFactory;
        $this->retailContextFactory = $retailContextFactory;
        $this->createService = $createService;
        $this->createRequestFactory = $createRequestFactory;
        $this->customerRepository = $customerRepository;
        $this->arrayOfCreateLineFactory = $arrayOfCreateLineFactory;
        $this->createHeaderFactory = $createHeaderFactory;
        $this->createLineFactory = $createLineFactory;
        $this->quoteRepository = $quoteRepository;
        $this->customerHelper = $customerHelper;
        $this->itemIdentifierFactory = $itemIdentifierFactory;
        $this->customerFactory = $customerFactory;
        $this->sellerRepository = $sellerRepository;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
    }

    /**
     * @param OrderInterface $order
     * @return CreateResponse
     * @throws LocalizedExceptionAlias
     * @throws NoSuchEntityException
     */
    public function createOrder(OrderInterface $order)
    {
        /** @var Create $createService */
        $createService = $this->createService->create();

        $header = $this->getHeader($order);
        $lines = $this->getLines($order, $header->getOrigin());

        /** @var Create_Request $request */
        $request = $this->createRequestFactory->create([
            'header' => $header
        ]);

        $request->setLines($lines);

        /** @var \FiloBlu\CegidY2\Api\SaleDocumentService\StructType\Create $parameters */
        $parameters = $this->createFactory->create();
        $parameters->setCreateRequest($request);
        $parameters->setClientContext($this->getRetailContext());
        if ($createService->Create($parameters) !== false) {
            return $createService->getResult();
        }

        throw new RuntimeException(implode("\n", $createService->getLastError()));
    }

    /**
     * @param OrderInterface $order
     * @return Create_Header
     * @throws LocalizedExceptionAlias
     * @throws NoSuchEntityException
     */
    public function getHeader(OrderInterface $order)
    {
        /** @var Create_Header $header */
        $header = $this->createHeaderFactory->create();
        $customerId = $order->getCustomerId();
        $customerCegidId = null;

        if ($customerId === null || (int)$order->getCustomerIsGuest() === 1) {
            /* GUEST  */
            $email = $order->getCustomerEmail();
            /** @var CustomerInterface $customer */
            $customer = $this->customerFactory->create();
            $customer->setEmail($email);

            $response = $this->customerHelper->searchCustomerByEmail($customer);

            if ($response->getSearchCustomerIdsResult()->getCustomerQueryData() === null) {
                $shippingAddress = $order->getShippingAddress();

                $customer->setLastname($shippingAddress->getLastName());
                $customer->setFirstname($shippingAddress->getFirstname());

                $customer = $this->customerHelper->createCustomer($customer);
                $customerCegidId = $this->getCompatibleCustomerCustomAttribute($customer,'cegid_id');
            } else {
                $customerCegidId = $response->getSearchCustomerIdsResult()->getCustomerQueryData()[0]->getCustomerId();
            }

        } else {

            $customer = $this->customerRepository->getById($order->getCustomerId());
            $cegidCustomerId = $this->getCompatibleCustomerCustomAttribute($customer,'cegid_id');
            if (!$cegidCustomerId) {
                $response = $this->customerHelper->searchCustomerByEmail($customer);

                if ($response === null) {
                    $customer->setLastname($customer->getLastname());
                    $customer->setFirstname($customer->getFirstname());
                    $customer = $this->customerHelper->createCustomer($customer);
                    $customerCegidId = $customer->getData('cegid_id');
                } else {
                    $customerCegidId = $response->getSearchCustomerIdsResult()->getCustomerQueryData()[0]->getCustomerId();
                }
            }
        }

        $header->setCurrencyId($order->getOrderCurrency()->getCode());
        $header->setCustomerId($customerCegidId);
        $header->setInternalReference($order->getIncrementId());
        $header->setExternalReference($order->getIncrementId());
        $header->setDate(substr($order->getUpdatedAt(), 0, 10));

        /*
        // TODO: Move this on a plugin
        // SHOP OR ECOMMERCE
        $builder = $this->searchCriteriaBuilderFactory->create();
        $builder->addFilter('main_table.entity_id', $order->getQuoteId(), 'eq');
        $result = $this->quoteRepository->getList($builder->create());

        if ($result->getTotalCount() > 0) {
            $sellerId = $result->getItems()[$order->getQuoteId()]->getExtensionAttributes()->getCegidSellerId();
            $seller = $this->sellerRepository->get($sellerId);
            $header->setSalesPersonId($sellerId);
            $header->setStoreId($seller->getShopCode());
        } else {
            $header->setStoreId('ECO01');
        }
        // END SHOP OR ECOMMERCE
        */

        $header->setTaxExcluded(false);
        $header->setType(SaleDocumentType::VALUE_RECEIPT);
        $header->setOrigin(DocumentOrigin::VALUE_SHOP);

        return $header;
    }

    /**
     * @param OrderInterface $order
     * @param $origin
     * @return ArrayOfCreate_Line
     */
    public function getLines(OrderInterface $order, $origin)
    {
        $lines = $this->arrayOfCreateLineFactory->create();

        foreach ($order->getItems() as $item) {
            if((int)$item->getParentItemId() > 0){
                continue;
            }
            $line = $this->createLineFactory->create();

            $identifier = $this->itemIdentifierFactory->create(['reference' => $item->getSku()]);
            $line->setItemIdentifier($identifier);
            $line->setOrigin($origin);
            $line->setNetUnitPrice($item->getPriceInclTax());
            $line->setQuantity($item->getQtyOrdered());
            $line->setUnitPrice($item->getBasePriceInclTax());
            $lines->addToCreate_Line($line);
        }

        return $lines;
    }

    /**
     * @return RetailContext
     */
    public function getRetailContext()
    {
        return $this->retailContextFactory->create(['databaseId' =>
            $this->credentialProvider->getDatabase('sale_document_service')
        ]);
    }

    // This function will work to retrieve custom customer attributes under Magento 2.2.x and 2.3.x
    private function getCompatibleCustomerCustomAttribute($customer,$custom_attribute_code){
        // This is for Magento 2.2.x
        $attribute_value = $customer->getCustomAttribute($custom_attribute_code);
        // Fix for Magento 2.3.x
        if (!is_string($attribute_value) && $attribute_value != null){
            $attribute_value = $customer->getCustomAttribute($custom_attribute_code)->getValue();
        }
        return $attribute_value;
    }
}